<?php
/* Logaholic Web Analytics software             Copyright(c) 2005-2011 Logaholic B.V.
 *                                                               All rights Reserved.
 * This code is subject to the Logaholic license. Unauthorized copying is prohibited.
 * support@logaholic.com                         http://www.logaholic.com/License.txt
*/ 
/**
* @desc This is the master report class. All other reports should be childen of this parent class
* it is assumed that this is included somewhere where common.inc.php is also included 
*/

include_once("includes/datavisualization.php");

class Report extends DataVisualization {
	var $sortTable;			   		// If this is set to false, no javascript datatable plugin will be activated
    var $help;                 		// This contains the help text for a report
    var $bchart;               		// An array containing the columns from showfields that should be displayed as a horizontal 'bar chart' in the table
	var $addlabel;					// this can be used to add custom text to the report options list in the header
	var $customHeaderContent;  		// replaces the list of report options in the header with a custom string (default empty)
    var $displayTotalRow;      		// controls if we want to display a row with totals at the bottom of a table (default true)
	var $graphcolors;          		// contains the colors of graphs and pies.
	var $template;			   		// Globaling the template
	var $displaymode;				// defines if you want to see the data as a table, pie, chart etc
	var $DefaultDisplay;
	var $period;
	var $roadto;
	var $sparktype;
	
    function __construct() {
        global $from, $to, $profile, $labels, $template, $db, $reports, $get_constant, $get_label;
		
        $this->from = $from;
        $this->to = $to;
        $this->profile = (isset($profile))? $profile : false;
        $this->bchart = array();
		$this->addlabel = "";
		$this->displayHeader = true;
		$this->displayReportLabel = false;
		$this->displayReportButtons = true;
		$this->displayTotalRow = true;
		$this->displayMaxChars = 100;
		$this->template = $template;
		$this->disableShowfieldsHandling = array();		
		$this->allowDateFormat = true;
		$this->DefaultDisplay = "table";
		$this->period = 'auto';
		$this->sparktype = "";
		$this->roadto = "";
		
		if($this->profile !== false) {
			if($this->profile->animate == 1) {
				$this->animate_graph = false;
			} else {
				$this->animate_graph = true;
			}
		}
		
		$useragent = $_SERVER['HTTP_USER_AGENT'];
		if(strpos($useragent, "MSIE") > 0) {
			$ie_agent = substr($useragent, strpos($useragent, "MSIE"), 10);
			$ie_agent = explode(" ", $ie_agent);
			$ie_version = explode(".", $ie_agent[1]);
			$ie_version = $ie_version[0];
		}
		
		if(!empty($ie_version) && $ie_version <= 8) {
			$this->graphcolors = array("rgb(0,120,174)","rgb(255,0,0)","rgb(0,111,0)",
			"rgb(0,221,34)","rgb(200,66,57)","rgb(249,113,247)",
			"rgb(60,31,110)","rgb(46,136,10)","rgb(221,24,64)",
			"rgb(252,218,0)","rgb(25,65,165)","rgb(175,216,248)",
			"rgb(246,189,15)","rgb(139,186,0)","rgb(166,110,221)",
			"rgb(249,132,161)","rgb(204,204,0)","rgb(153,153,153)",
			"rgb(0,153,225)","rgb(255,102,204)","rgb(102,153,102)",
			"rgb(124,124,180)","rgb(255,153,51)","rgb(152,0,255)",
			"rgb(153,0,204)","rgb(204,204,0)","rgb(102,153,0)",
			"#FF0000","#00F000","#0000F0","#FF00FF","#00FFFF",
			"#FFFF00","#1970B4","#19B470","#7019B4","#70B419",
			"#B41970","#B47019","#FB9619","#FB1996","#96FB19",
			"#9619FB","#1996FB","#19FB96","#CC0000","#00C000",
			"#0000C0","#CC00CC","#00CCCC","#CCCC00","#414D91",
			"#41914D","#4D4191","#4D9141","#91414D","#914D41",
			"#E78741","#E74187","#87E741","#8741E7","#4187E7",
			"#41E787","#990000","#009000","#000090","#990099",
			"#009999","#999900","#692A6E","#696E2A","#2A696E",
			"#2A6E69","#6E692A","#6E2A69","#D37869","#D36978",
			"#78D369","#7869D3","#6978D3","#69D378","#660000",
			"#006000","#000060","#660066","#006666","#666600",
			"#91074B","#914B07","#07914B","#074B91","#4B9107",
			"#4B0791","#BF6991","#BF9169","#69BF91","#6991BF",
			"#9169BF","#91BF69");
		} else {		
			
			$this->graphcolors = array("rgba(226,82,29,1)","rgba(119,189,57,1)","rgba(44,181,233,1)","rgba(233,165,38,1)",
			"rgba(0,120,174,1)","rgba(255,0,0,1)","rgba(0,111,0,1)",
			"rgba(0,221,34,1)","rgba(200,66,57,1)","rgba(249,113,247,1)",
			"rgba(60,31,110,1)","rgba(46,136,10,1)","rgba(221,24,64,1)",
			"rgba(252,218,0,1)","rgba(25,65,165,1)","rgba(175,216,248,1)",
			"rgba(246,189,15,1)","rgba(139,186,0,1)","rgba(166,110,221,1)",
			"rgba(249,132,161,1)","rgba(204,204,0,1)","rgba(153,153,153,1)",
			"rgba(0,153,225,1)","rgba(255,102,204,1)","rgba(102,153,102,1)",
			"rgba(124,124,180,1)","rgba(255,153,51,1)","rgba(152,0,255,1)",
			"rgba(153,0,204,1)","rgba(204,204,0,1)","rgba(102,153,0,1)",
			"#FF0000","#00F000","#0000F0","#FF00FF","#00FFFF",
			"#FFFF00","#1970B4","#19B470","#7019B4","#70B419",
			"#B41970","#B47019","#FB9619","#FB1996","#96FB19",
			"#9619FB","#1996FB","#19FB96","#CC0000","#00C000",
			"#0000C0","#CC00CC","#00CCCC","#CCCC00","#414D91",
			"#41914D","#4D4191","#4D9141","#91414D","#914D41",
			"#E78741","#E74187","#87E741","#8741E7","#4187E7",
			"#41E787","#990000","#009000","#000090","#990099",
			"#009999","#999900","#692A6E","#696E2A","#2A696E",
			"#2A6E69","#6E692A","#6E2A69","#D37869","#D36978",
			"#78D369","#7869D3","#6978D3","#69D378","#660000",
			"#006000","#000060","#660066","#006666","#666600",
			"#91074B","#914B07","#07914B","#074B91","#4B9107",
			"#4B0791","#BF6991","#BF9169","#69BF91","#6991BF",
			"#9169BF","#91BF69");
		}
		
		# set $_GET to vars 
		foreach($_GET as $key => $value) {
			if(isset($value)) {
				$this->$key = addslashes($value);
				$this->options[$key] = addslashes($value);
			}
		}
		
		# set $_POST to vars 
		foreach($_POST as $key => $value) {
			if(isset($value)) {
				$this->$key = addslashes($value);
				$this->options[$key] = addslashes($value);
			}
		}
		
		if(empty($this->source)) {
			$this->source = "";
		}
		
		if($this->period == 'auto') {
			$this->period = $this->getPeriod($this->from, $this->to);
		}	
		
		if(!isset($this->trafficsource)) {
			$this->trafficsource = null;
			$this->applytrafficsource = false;
		} else {
			$this->applytrafficsource = true;
		}
		
		if(empty($this->limit) || is_numeric($this->limit) === false) {
			$this->limit = 10;
		}
		
		if($this->limit <= 20) {
			$this->paginationPanel = false;
		} else {
			$this->paginationPanel = true;
		}
		
		if(empty($this->current_plotting_graph)) {
			$this->current_plotting_graph = 1;
		}
		
		$this->sortTable=true;
		
        if(defined($labels)) {
            $labels = constant($labels);
        }
        $this->label = $labels;
		
		if(!empty($this->label) && empty($this->roadto)) {
			if (defined($this->label)) {
				$opts = $reports[$this->label]['Options'];
			} else {
				$opts = $reports[$get_constant[$this->label]]['Options'];
			}
			if(strpos($opts, 'roadto') !== false) {
                $targetfiles = explode(",", str_replace(" ", "", $this->profile->targetfiles));
				if(!empty($targetfiles[0])) {
					$this->roadto = $targetfiles[0];
				} else {
					$this->roadto = '';
				}
            }
        }
		
		if(method_exists($this, "Settings") == true) {
			$this->Settings();
		}
		
		if(empty($this->displaymode)) {
			$this->displaymode = $this->DefaultDisplay;
		}
		
		if(empty($this->label)) {			
			if(isset($get_label[get_class($this)])){
				$this->label = constant($get_label[get_class($this)]);
			}
		}
    }
    # do this for PHP 4 compatibility
    function Report() { __construct(); }
    
    # this function displays any custom form fields needed to run this report
    function DisplayCustomForm() {
        return false;
    }
    
    # this function returns either a database query or false
    function DefineQuery() {
        return false;
    }
    
    # this function creates a data array that we will use to display results
    function CreateData() {
        global $db;
        $q = $this->DefineQuery();
        if ($q!==false) {
            if ($this->applytrafficsource == true) { $q = subsetDataToSourceID($q,$this->trafficsource);  }
            $db->SetFetchMode(ADODB_FETCH_NUM);
            $result = $db->Execute($q);
            $data = $result->GetArray();
            $db->SetFetchMode(ADODB_FETCH_BOTH);
            return $data;
        } else {
            # if the report is not based on a single database query, this is where the data will be created
            return false;
        }
    }
    
    # this function will print the report, by default it just prints a table, but this can be overruled in child classes
    function DisplayReport() {
        $data = $this->CreateData();
		
		if(isset($this->DefaultDisplay) && !isset($this->displaymode)) { $this->displaymode = $this->DefaultDisplay; }
		
		if(isset($this->displaymode) && !empty($data)) {
			if($this->displaymode == "pie") {
				$this->displayReportButtons = false;
				
				$fields = explode(",",$this->showfields);
				$fields = $fields[0].",".$fields[1];
				$this->showfields = $fields;
				
				$this->reportHeader();				
				$this->PieChart($data);
				
			} elseif($this->displaymode == "linechart") {
				$this->displayReportButtons = false;
		
				$this->reportHeader();				
				//$this->Graph($data);
				$this->Graph($data,'line', '', 0, 250, @$this->custom_graphoptions);
				
				
			} elseif($this->displaymode == "barchart") {
				$this->displayReportButtons = false;
		
				$this->reportHeader();				
				//$this->Graph($data, 'bar');
				$this->Graph($data,'bar', '', 0, 250, @$this->custom_graphoptions);
				
				
			} elseif($this->displaymode == "areachart") {
				$this->displayReportButtons = false;
		
				$this->reportHeader();				
				//$this->Graph($data, 'area');
				$this->Graph($data,'area', '', 0, 250, @$this->custom_graphoptions);
				
				
			} elseif($this->displaymode == "bubble") {
				$this->displayReportButtons = false;
		
				$this->reportHeader();				
				$this->BubbleChart($data);
				
			} elseif($this->displaymode == "stackedbar") {
				$this->displayReportButtons = false;
		
				$this->reportHeader();				
				//$this->Graph($data, 'stackedbar');
				$this->Graph($data,'stackedbar', '', 0, 250, @$this->custom_graphoptions);
				
			} elseif($this->displaymode == "barlinechart") {
				$this->displayReportButtons = false;
		
				$this->reportHeader();				
				$this->Graph($data, $this->custom_graphtype, '', 0, 250, @$this->custom_graphoptions, 'south', 'inline');
				
			} else {
				$this->Table($data);
			}
		} else {
			$this->Table($data);
		}
    }
	
	
	public function ReorderColumns($data,$neworder) {
		if (empty($neworder)) {
			return $data;
		}		
		$cols = array_flip(explode(',',$this->showfields));
		$order = explode(',',$neworder);
		$ncols = count($order);
		$newdata = array();
		$i=0;
		foreach ($data as $row) {
			$c = 0;
			while ($c < $ncols) {
				$thiscol = $cols[$order[$c]];
				$newdata[$i][$c] = $row[$thiscol];
				$c++;
			}
			$i++;
		}
		return $newdata;
	}
	
	public function CompareMeter($what,$val_a,$val_b,$text_a="",$text_b="") {
				
		if ($val_a == 0 && $val_b == 0) {
			$a=50;
			$b=50;
		} else {		
			$tot = ($val_a*100) + ($val_b*100);
			$a = round((($val_a*100) / $tot) * 100);
			$b = round((($val_b*100) / $tot) * 100);		
		}	
		if (empty($text_a)) {
			$text_a=$val_a;
			$text_b=$val_b;			
		}
		
		echo "<div class='compare_what' style='margin:0 auto;width:500px;font-style:italic;text-align:center;padding:3px 0 3px 0;'>$what</div>";
		echo "<div style='margin:0 auto;width:500px;border:1px solid grey;border-radius:4px;clear:both;margin-bottom:3px;'>";
		
		echo "<div style='position:absolute;width:250px;height:20px;border-right:1px solid white;'></div>";
		
		echo "<div class='compare_left' style='font-size:11px;width:$a%;height:20px;line-height:20px;background: #de241b url(images/bg_glass_red2.png) 50% 50% repeat-x;color:white;float:left;'></div>";		
		echo "<div class='compare_right' style='font-size:11px;width:$b%;height:20px;line-height:20px;background: #49c106 url(images/bg_glass_green.png) 50% 50% repeat-x;float:right;text-align:right;'></div>";
		echo "<div style='float:left;margin-top: -20px;width:100%; line-height:20px;'>";
		echo "<p class='compare_left_text' style='float:left; margin:0 0 0 5px; padding:0; color:#FFFFFF;'>".$text_a."</p>";
		echo "<p class='compare_right_text' style='float:right; margin:0 5px 0 0; padding:0; color:#000000;'>".$text_b."</p>";
		echo "</div>";
		echo "<div class=clear></div>";
		echo "</div>";
		
	}
	
	function DisplayWinner($data1,$data2,$itemname=_PAGE){		
		$confidence = 0;
		$cfactor = 0;
		$md5_time = md5(time());
		$container_id = $md5_time.rand(1,1000);
			
		echo "<div id='decide_winner'>";
		echo "<div id='$container_id' class='confidence_graph'></div>";
		
		if ($data1["page_total"]==0 || $data2["page_total"]==0) {
			echo "<b>"._NOT_ENOUGH_DATA_TO_DETERMINE."!</b><br/><br/>";
		} else {
			$epop = ($data1["target_indirect_total"] + $data2["target_indirect_total"]) / ($data1["page_total"] + $data2["page_total"]);
			$eerr = sqrt(($epop * (1 - $epop) * ($data1["page_total"] + $data2["page_total"]))/($data1["page_total"] * $data2["page_total"]));
			$cfactor = @(abs((@($data1["target_indirect_total"]/$data1["page_total"]) - @($data2["target_indirect_total"]/$data2["page_total"])))/$eerr);
			
			if ($cfactor > 0.01) {
			 $confidence=1;
			}
			if ($cfactor > 0.06) {
			 $confidence=5;
			}
			if ($cfactor > 0.14) {
			 $confidence=10;
			}
			if ($cfactor > 0.25) {
			 $confidence=20;
			}
			if ($cfactor > 0.52) {
			 $confidence=39;
			}
			if ($cfactor > 0.68) {
			 $confidence=50;
			}
			if ($cfactor > 1.00) {
			 $confidence=68;
			}
			if ($cfactor > 1.64) {
			 $confidence=90;
			}
			if ($cfactor > 1.96) {
			 $confidence=95;
			}
			if ($cfactor > 2.58) {
			 $confidence=99;
			}
			if ($cfactor > 2.81) {
			 $confidence=99.5;
			} 

			if (($data1["target_indirect_total"]/$data1["page_total"]) > ($data2["target_indirect_total"]/$data2["page_total"])) {
					 $winner="<span class=testa>$itemname A</span>";
					 $loser="<span class=testb>$itemname B</span>";
			} else {
					 $winner="<span class=testb>$itemname B</span>";
					 $loser="<span class=testa>$itemname A</span>";
			}
			if ($confidence >= 90) {
				echo "<font size=+1>"._SPLIT_TEST_WINNER." <b>$winner</b>!</font><br/><br/>";
				echo _TEST_RESULTS_ARE_SIGNIFICANT." ($confidence% "._CONFIDENCE."). "._YOU_CAN_BE_ABOUT." $confidence% "._CONFIDENT_THAT." $winner "._WILL_PERFORM_BETTER_THAN." $loser";
				echo "<br>";
			} else if ($confidence > 50) {
				echo "<font size=+1>"._SPLIT_TEST_WINNER." <b>$winner</b></font>";
				echo "<P><b>$winner performed better but ... </b></p>";
				echo _TEST_RESULTS_ARE_NOT_SIGNIFICANT." ($confidence% "._CONFIDENCE."). "._WHEN_THE_LEVEL_DROPS." $winner didn't "._JUST_GOT_LUCKY.".";
				echo "<br>";
			} else {
				echo "<font size=+1>"._SPLIT_TEST_WINNER." "._INCONCLUSIVE."</font>";
				echo "<P><b>$winner performed better but ... </b></p>";
				echo _TEST_RESULTS_ARE_NOT_SIGNIFICANT." ($confidence% "._CONFIDENCE."). "._WHEN_THE_LEVEL_DROPS." $winner did not just perform better by chance."; //"._JUST_GOT_LUCKY.".";
				echo "<br>";
			}
		}
		echo "</div>";
		?>
		<script type="text/javascript" charset="utf-8">
		$(document).ready(function(){
			s1 = [<?php echo $confidence;?>];
	 
			plots['<?php echo $container_id; ?>']  = $.jqplot('<?php echo $container_id;?>',[s1],{
			    seriesDefaults: {
				   renderer: $.jqplot.MeterGaugeRenderer,
				   rendererOptions: {
					   label: 'Confidence',
					   padding:0,
					   min: 0,
					   max: 100,
					   intervals:[25, 50, 75, 90, 100],
					   intervalColors:['#FCF2E5','#FCDCB3','#FCB85F','#FC950F','#0078AE']
				   }
			    },
			    highlighter: {
					show:false
				},
				cursor: {
					show:false
				},
				
			});
		});
		</script>
		<?php
			
	}
	
	function ReportHeaderButtons() {
		global $cachename, $validUserRequired;
		

		//echo "<a target='_blank' href='reports.php?conf={$this->profile->profilename}&to={$this->to}&from={$this->from}&labels={$this->labels}&limit={$this->limit}&outputmode=print' title=\""._PRINTABLE_REPORT."\"><img src='images/icons/printer.png'></a>";
		echo "<a target='_blank' href='reports.php?{$_SERVER['QUERY_STRING']}&outputmode=print' title=\""._PRINTABLE_REPORT."\"><img src='images/icons/printer.png'></a>";
		
		// echo "<a target='_blank' href='reports.php?{$_SERVER['QUERY_STRING']}&outputmode=xml' title=\""._PRINTABLE_REPORT."\">XML</a>";
		
		echo "<a class='mail_btn' title=\""._EMAIL_REPORT."\"><img src='images/icons/email_go.gif'></a>";

		echo "<a href='reports.php?{$_SERVER['QUERY_STRING']}&outputmode=csv' target='_blank' title=\""._SAVE_REPORT_CSV."\"><img src='images/icons/csvexport.gif'></a>";
		
		echo "<a class='help_btn' title='Open Help'><img src='images/icons/help.png'></a>";
		echo "<div class='clear'></div>";
		
		if (!$validUserRequired && (!isset($_SERVER["PHP_AUTH_USER"]))) {
			//echo "<div id=\"mailer{$cachename}\" style=\"display : none; line-height : 18px; position : absolute; top:100px;left:250px;background-color:#f0f0f0;border: 2px solid red;z-index:10;padding:15px;\"><img src=\"images/icons/email_go.gif\" alt=\"\" align=left> "._ONLY_WHEN_PASSWORD_PROTECTED."</div>";
			echo "<div id=\"mailer{$cachename}\" class='mailer' style='padding:10px;'><img src=\"images/icons/email_go.gif\" alt=\"\" align=left> "._ONLY_WHEN_PASSWORD_PROTECTED."</div>";
		} else {
			?>
			<div id="mailer<?php echo $cachename;?>" class="mailer">
				<form method=post action="reports.php?<?php echo $_SERVER['QUERY_STRING']; ?>&outputmode=email" target=_blank>
					<table cellpadding=5 cellspacing=5 width=450>
						<tr><td colspan=2 class=smallborder>
							<a class='mail_btn'><img src=images/icons/cancel.gif border=0 align=right></a><img src=images/icons/email_go.gif width=16 height=16 align=left> &nbsp;&nbsp;<b><?php echo _SEND_THIS_REPORT_AS_EMAIL;?>:</b>
							<?php //  href="javascript:mailbox('mailer<?php echo $cachename;  ')" ?>
						</td></tr>
						<tr><td title="<?php echo _EMAIL_FROM;?>"><?php echo _YOUR_EMAIL;?>:</td><td title="<?php echo _EMAIL_FROM;?>"> <input type=text name=fromemail size=40></td></tr>
						<tr><td title="<?php echo _EMAIL_TO;?>"><?php echo _THEIR_EMAIL;?>:</td><td title="<?php echo _EMAIL_TO;?>"> <input type=text name=email size=40></td></tr>
						<tr><td><?php echo _SUBJECT;?>:</td><td> <input type=text name=subject value="<?php  echo $this->profile->confdomain . " " .$this->label; ?>" size=40></td></tr>
						<tr><td><?php echo _MESSAGE;?>:</td><td> <textarea name=message cols=40 rows=7></textarea></td></tr>
						<tr><td></td><td> 
						<input type=submit value="Send" onclick="javascript:mailbox('mailer<?php echo $cachename;?>')"><p>
						<?php echo _YOU_WILL_RECEIVE_COPY_OF_EMAIL;?></p>
						</td></tr>
					</table>
				</form>        
			</div>
			<?php
		}

	}
	
	function UpdateStats(){
		return false;
	}	
	
	function DisplayCSV() {
        $data = $this->CreateData();
        $this->CSVStatsTable($data,$this->from,$this->to,$this->showfields,$this->label);
    }
	
	function DisplayXML() {
        $data = $this->CreateData();
        $this->XMLStatsTable($data,$this->from,$this->to,$this->showfields,$this->label);
    }
	
	function DisplaySimpleTable() {
        $data = $this->CreateData();
        $this->SimpleStatsTable($data,$this->from,$this->to,$this->showfields,$this->label);
    }	
	
	function MakeSearchString($search,$field,$searchmode) {
		//this function turns the search field input into valid sql
		
		if (strpos($search," and ")!=FALSE) {
			$searchitems=explode(" and ", $search);
			$andor="AND";
		} else if (strpos($search," or ")!=FALSE) {
			$searchitems=explode(" or ", $search);
			$andor="OR";
		} else {
			$searchitems[0]= $search;
			$andor="AND";
		}
		$i=0;
		$searchst="and ($field $searchmode '%$searchitems[$i]%' ";
		$i++;
		while (@$searchitems[$i]!="") {
			$searchst.="$andor $field $searchmode '%$searchitems[$i]%' ";
			$i++;
		}
		$searchst.=")";
		return $searchst;    
	}

	function SearchMatchingIDs($search,$field,$searchmode,$tables) {
		global $db;
		//this function makes a subquery that returns matching id's
		
		if (strpos($search," and ")!=FALSE) {
			$searchitems=explode(" and ", $search);
			$andor="AND";
		} else if (strpos($search," or ")!=FALSE) {
			$searchitems=explode(" or ", $search);
			$andor="OR";
		} else {
			$searchitems[0]= $search;
			$andor="AND";
		} 
		$i=0;
		$searchst="($field $searchmode '%$searchitems[$i]%' ";
		$i++;
		while (@$searchitems[$i]!="") {
			$searchst.="$andor $field $searchmode '%$searchitems[$i]%' ";
			$i++;
		}
		$searchst.=")";
	   
		$searchst = "and $field IN (select id from $tables where $searchst)";
		return $searchst;    
	}
	
	// this creates an empty zerofilled array
	function newReportArray($rows,$collums) {
		$data = array();
		$r=0;
		while ($r < $rows) {
			$c=0;
			while ($c < $collums) {
				$data[$r][$c] = 0;
				$c++;
			}
			$r++;
		}
		return $data;
	}
	
	// this function takes a series based array and converts/pivots the array to columns
	function seriesToColumns($input_data, $row_key=0, $row_label="Date", $series_key=1) {
		$data = array();
		if(empty($input_data)){
			return false;
		}
		# create a column name array, the first element should be the row label
		$col_names = array();
		$col_names[]=$row_label;		
				
		# now get the column names
		foreach ($input_data as $row => $serie_data) {
			if (!in_array($serie_data[$series_key],$col_names)) {
				$col_names[] = $serie_data[$series_key];
			}
		}		
		$col_nums = array_flip($col_names);
		$cn = count($col_nums);
		
		# convert the input data
		foreach ($input_data as $row => $serie_data) {
			# use the chosen field from the serie data as the row key for the new array
			$rk = $serie_data[$row_key];
			
			# use the chosen field from the serie data as column key
			$ck = $col_nums[$serie_data[$series_key]];
			
			# loop each series array and transform to column
			foreach ($serie_data as $col_num => $val) {
				# if column is the series key, skip it
				if ($col_num == $series_key) { 
					continue; 
				}
				# add a column to the array.
				if ($col_num == $row_key) {
					$data[$rk][$row_key] = $val;
				} else {
					$data[$rk][$ck] = $val;
				}
			}			
			# fill empty data
			for ($i=0;$i<$cn;$i++) {
				if (!isset($data[$rk][$i])) { $data[$rk][$i]=0; }
			}
		}
		$result['fields'] = $col_names;
		$result['data'] = $data;
		return $result;		
	}
	
	// this function returns the number of intervals between from and now, based on the period
	function dateNumber($from, $now, $period) {
		switch ($period) {
			case _DAYS: return round(($now - $from) / 86400);
			case _WEEKS: return round(($now - $from) / (7*86400));
			case _MONTHS: return round(($now - $from) / (31*86400));
        }
	}
	
	// this function returns the date period to use 
	function getPeriod($from, $to) {
		if((($to - $from) / 86400) >= 217) {
			$period = _MONTHS;
		} elseif((($to - $from) / 86400) >= 32) {
			$period = _WEEKS;
		} else {
			$period = _DAYS;
		}
		return $period;
	}
	
	// this function retuns the number of seconds in an interval period
	function getSeconds($period) {
		switch ($period) {
			case _DAYS: return 86400;
			case _WEEKS: return (7*86400);
			case _MONTHS: return (31*86400);
        }
		return false;
	}
	
	// this function retuns the default date anotation of an interval period
	function getFormatDate($period,$timestamp) {
		switch ($period) {
			// case _DAYS: return  date("D, m/d/Y",$timestamp);
			case _DAYS: return  date(implode($this->profile->dateFormat),$timestamp);
			case _WEEKS: return date('o-\\WW',$timestamp);
			case _MONTHS: return date("M Y",$timestamp);
        }
		return false;
	}
	
	// This function must be use in DefineReport()
	function externDbConnectForm($savename){
		// Checking global settings if data is already stored.
		$connection = unserialize(getProfileData($this->profile->profilename,$this->profile->profilename.".externalConnection.$savename",""));
		// if  not stored setup the array but empty.
		if(empty($connection)){
			$connection["server"] = "";
			$connection["username"] = "";
			$connection["password"] = "";
			$connection["database"] = "";
			$connection["tableprefix"] = "";
		}
		// if saved store data.
		if($this->action == "save"){
			echoNotice("Saved your connection data.","margin:15px;");
			$connection["server"] = $_REQUEST["server"];
			$connection["username"] = $_REQUEST["username"];
			$connection["password"] = $_REQUEST["password"];
			$connection["database"] = $_REQUEST["database"];
			$connection["tableprefix"] = $_REQUEST["tableprefix"];
			$save = serialize($connection);
			setProfileData($this->profile->profilename,$this->profile->profilename.".externalConnection.$savename",$save);
			setProfileData($this->profile->profilename,$this->profile->profilename.".externalConnection.prefix.$savename",$connection["tableprefix"]);
		}
		$connectionForm = "<form method=POST>";
		$connectionForm .= "<table>";
		$connectionForm .= "<tr>";
		$connectionForm .= "<td><label for='server'>Server:</label></td>";
		$connectionForm .= "<td><input type='text' id='server' name='server' value='".$connection["server"]."' /></td>";
		$connectionForm .= "</tr><tr>";
		$connectionForm .= "<td><label for='server'>Username:</label></td>";
		$connectionForm .= "<td><input type='text' id='username' name='username' value='".$connection["username"]."' /></td>";
		$connectionForm .= "</tr><tr>";
		$connectionForm .= "<td><label for='server'>Password:</label></td>";
		$connectionForm .= "<td><input type='password' id='password' name='password' value='".$connection["password"]."' /></td>";
		$connectionForm .= "</tr><tr>";
		$connectionForm .= "<td><label for='server'>Database:</label></td>";
		$connectionForm .= "<td><input type='text' id='database' name='database' value='".$connection["database"]."' /></td>";
		$connectionForm .= "</tr><tr>";
		$connectionForm .= "<td><label for='tableprefix' title='Leave empty if not set'>Table Prefix:</label></td>";
		$connectionForm .= "<td><input type='text' id='tableprefix' name='tableprefix' value='".$connection["tableprefix"]."' /></td>";
		$connectionForm .= "</tr><tr>";
		$connectionForm .= "<td></td><td><input type='submit' value='Set Connection' /></td>";
		$connectionForm .= "</tr>";
		$connectionForm .= "</table>";
		$connectionForm .= "<input type='hidden' name='action' value='save' />";
		$connectionForm .= "<form>";
		echo "<div style='height:100%; float:left; width:50%;'>";
		echo "<h2>Setup connection data.</h2>";
			echo $connectionForm;
		echo "</div>";
		echo "<div style='height:100%; border-left:1px solid silver; float:left; width:40%; padding:0 10px;'>";
			echo "<h2>Testing your connection.</h2>";
			
			if($con = $this->checkExternalConnection($savename)){			
				echo "<p>Connected to server.</p>";
				echo "<p>Connected to database.</p>";
				echo "<p>You are now ready to use this connection.</p>";
				echo "<p>Please close this frame.</p>";
				mysql_close($con);
			}else{
				echo "<p>Could not connect to server or database.</p>";
				echo "<p>Please check your settings.</p>";
			}
		echo "</div>";
	}	
	function checkExternalConnection($savename){
		/// From here we are testing the connection with the given data.
		$connection = unserialize(getProfileData($this->profile->profilename,$this->profile->profilename.".externalConnection.$savename",""));
		if(empty($connection)){
			$connection["server"] = "";
			$connection["username"] = "";
			$connection["password"] = "";
			$connection["database"] = "";
		}
		if(empty($connection["server"])){
			return false;
		}
		// Check connection with server
		$con = @mysql_connect($connection["server"],$connection["username"],$connection["password"]);
		if ($con == false) {
			return false;
		}
		// Check connection with database
		$db = mysql_select_db($connection["database"], $con);
		if ($db == false) {
			return false;
		}
		return $con;
	}
	
	# This function populated the graph color array.
	# Times is the amount of times the graph color array should multiply itself.
	function PopulateColors($colors = array(), $times = 2) {
		if(empty($colors)) {
			$colors = $this->graphcolors;
		}
		
		$newcolors = array();
		for($c = 1; $c <= $times; $c++) {
			foreach($colors as $color) {
				$newcolors[] = $color;
			}
		}
		
		return $newcolors;
	}
	
	function GetHiddenLegends($legends = array()) {
		# Check which legend items need to be hidden.
		# Returns and array containing the number of the corresponding legend items.
		
		$hiddenLegends = @$this->hiddenlegends;
		if(!empty($hiddenLegends)) {
			$hiddenLegends = json_decode($hiddenLegends);
		} else {
			$hiddenLegends = array(); 
		}
		
		return $hiddenLegends;
	}
	
	# a sparkline is a mini graph
	function DisplaySparkline() {
        $data = $this->CreateData();
        $this->Sparkline($data,$this->sparktype);
    }
	
	# this function will take a default data array and convert it to something that
	# we can use to generate sparkline graphs
	function ConvertToSparklineData($data) {
		$d = array();
		$dd = array();
		foreach ($data as $key => $val) {
			$d[] = "$key: '{$val[0]}'";
			$dd[] = $val[1];
		}
		$newdata['names'] = implode(", ",$d);
		$newdata['values'] = implode(", ",$dd);
		return $newdata;
	}
	
	function Sparkline($data, $type="", $label="", $postfix="") {
		global $get_constant;
				
		if (empty($label)) {
			$label = $this->label;
		}
		$clabel = $get_constant[$this->label];
		$sourcedata = $data;
		
		if ($postfix=="") {
			$showfield = explode(",",$this->showfields);
			$showfield = $showfield[1];
			$showvalue = $data;
			$showvalue = array_pop($showvalue);
			$showvalue = $showvalue[1];
			$trend = number_format(getTrend($data,1),0);
			if ($trend > 0) {
				$trend = "<span class=\'uptrend\'>+$trend</span>";
			} else {
				$trend = "<span class=\'downtrend\'>$trend</span>";
			}
			$postfix = "$showvalue $showfield ($trend)";
		}
		
		$data = $this->ConvertToSparklineData($data);
		
		$div_id = md5($label.time());
		
		switch($type) {

			case "bar":
			
				?>
					if ($("#<?php echo $clabel; ?>").length == 0){
						$('<div id ="<?php echo $clabel; ?>"></div>').appendTo(".inpage_container");
					}
					$('<div class="spark_container"><span><?php echo $label; ?>: <span id="<?php echo $div_id; ?>"><?php echo $data["values"]; ?></span></span></div>').appendTo("#<?php echo $clabel; ?>");
					$(function() {
						$("#<?php echo $div_id;?>").sparkline('html', {
							type: 'bar',
							tooltipFormat: '<span style="color: {{color}}">&#9679;</span> {{offset:names}}  ({{value}})',
							tooltipValueLookups: { names: { <?php echo $data['names'];?> } }							
						});
					});
					
				<?php
				break;
				
			case "pie":
			
				?>
					if ($("#<?php echo $clabel; ?>").length == 0){
						$('<div id ="<?php echo $clabel; ?>"></div>').appendTo(".inpage_container");
					}
					$('<div class="spark_container"><span><?php echo $label; ?>: <span id="<?php echo $div_id; ?>"><?php echo $data["values"]; ?></span> <?php echo $postfix; ?></span></div>').appendTo("#<?php echo $clabel; ?>");
					$(function() {
						$("#<?php echo $div_id;?>").sparkline('html', {
							type: 'pie',
							tooltipFormat: '<span style="color: {{color}}">&#9679;</span> {{offset:names}} ({{percent.1}}%)',
							tooltipValueLookups: { names: { <?php echo $data['names'];?> } }														
						});
					});					
				<?php
				break;
				
			default:
				?>
					if ($("#<?php echo $clabel; ?>").length == 0){
						$('<div id ="<?php echo $clabel; ?>"></div>').appendTo(".inpage_container");
					}
					$('<div class="spark_container"><span><?php echo $label; ?>: <span id="<?php echo $div_id; ?>"><?php echo $data["values"]; ?></span> <?php echo $postfix; ?></span></div>').appendTo("#<?php echo $clabel; ?>");
					$(function() {
						$("#<?php echo $div_id;?>").sparkline('html', {
							tooltipFormat: '<span style="color: {{color}}">&#9679;</span> {{offset:names}}  ({{y}})',
							tooltipValueLookups: { names: { <?php echo $data['names'];?> } }							
						});
					});
					

				<?php
				break;		
		}
		return $sourcedata;
	}
}
if (file_exists("includes/adwords_core.php")) {
	include_once("includes/adwords_core.php");
}
include_once("includes/twitter_core.php");

$ignore_report_files = array("test_center.php");

# THIS IS FOR TESTING THE REPORT STORE ... KILLS ALL THE REPORTS FROM THE REPORTS FOLDER! BEWARE!
if(isset($_GET["kill_reports"])){
	$delete_reports = getReportArray();
	foreach($delete_reports as $removal){	
		if ($removal["Distribution"]!="Standard") {
			if (file_exists("reports/".$removal["Filename"].".php")) {
				unlink("reports/".$removal["Filename"].".php");
			}
		}
	}
	unset($delete_reports);
}

# This is as good a place as any to include all the report child classes from the reports dir

#check if there is a user-login required, if the user has logged in and if the user is an admin.
if($validUserRequired && !empty($session->usersessionid) && !$session->isAdmin()){
	#if the current user is not an admin check if there is an subscripion set for this user.
	@$result = $db->Execute("SELECT subscriptionid FROM ".TBL_USERS." WHERE userid={$session->userinfo["userid"]}");
	
	#add a extra check to see if the user table has the subscription column added to it.
	if($result == false){
		$subscid = null;
	} else {
		$row = $result->fetchRow();
		$subscid = $row['subscriptionid'];
	}	
	# if there is an subscription set for the user get the report ids.
	if($subscid != null){
		$query = $db->Execute("SELECT value FROM ".TBL_SUBSCRIPTIONS." WHERE id={$subscid}");
		$row = $query->fetchRow();
		$subvalue = unserialize($row['value']);
		if(!empty($subvalue["reports"])){		
			foreach($subvalue["reports"] as $report_id){
				$q = $db->Execute("SELECT filename FROM ".TBL_REPORTS." WHERE id={$report_id}");
				$row = $q->fetchRow();
				$filename = $row['filename'];
				include_once(logaholic_dir() . "reports/".$filename.".php");
			}
		# if there are no reports set. Set an empty reports array.
		} else { $reports = array(); }
	# if the user has no subscription set for it then do the old report method.
	} else {
		# else we use the old include report method.
		$report_dir = opendir(logaholic_dir() . "reports/");
		while ($file = readdir($report_dir)) {
			if (strpos($file,".php")!==false) {
				if(in_array($file,$ignore_report_files) == false){
					include_once "reports/" . $file;
				}
			}
		}
		closedir($report_dir);
	}	
} else {
	# else we use the old include report method.
	$report_dir = opendir(logaholic_dir() . "reports/");
	while ($file = readdir($report_dir)) {
		if (strpos($file,".php")!==false) {
			if(in_array($file,$ignore_report_files) == false){
				include_once "reports/" . $file;
			}
		}
	}
	closedir($report_dir);
}


if (isset($profile)) {
	$downloaded_reports = getProfileData($profile->profilename, "{$profile->profilename}.downloaded_reports", "");
	$is_download = array();
	if(!empty($downloaded_reports)) {
		$downloaded_reports = unserialize($downloaded_reports);
		
		foreach($downloaded_reports as $downloaded_report) {
			if (!file_exists("reports/".$downloaded_report.".php")) {
				$dld_report_data = unserialize(getProfileData($profile->profilename, "{$profile->profilename}.report.{$downloaded_report}"));
				if(!empty($dld_report_data['expires_after'])) {
					if($dld_report_data['installDate'] + $dld_report_data['expires_after'] > time()) {
						$not_expired = true;
					} else {
						$not_expired = false;
					}
				} else {
					$not_expired = true;
				}
				if($not_expired == true) {
					$dl_report = base64_decode($dld_report_data['code']);
					$dl_report = "?>".$dl_report;
					//var_dump($dld_report_data);
					// echo "<pre>".$dl_report."</pre>";
					eval($dl_report);
					$is_download[$dld_report_data['classname']] = $downloaded_report;
				}
			}
		}
	}
}

// sort out the reports
$reports = sort_reports($reports);

#Now we need to make a label/constant matcher, so can always get the constant name
$get_constant = array();
foreach ($reports as $key => $value) {
    if (defined($key)) {
        $get_constant[constant($key)] = $key;
		$get_label[$value['ClassName']] = $key;
    }
}

?>
